% Test the projection procedure. Generates a random problem and samples in
% the resulting ranges of (squared) distances. The projects the sampled
% problem (the projection for point problems is easy) and computes the
% min/max values for the projection. These values are compared with those
% obtained with the projection procedure.
% The first part of this function can be tuned to select the kind of
% experiment to be performed.
%
% Inputs:
%     S0: The random seed. Optional. This is used to repeat the experiments
%         when an error is detected.
% Outputs:
%      e: The relative error in the projection.
%      S: The random seed.
%
function [e,S]=testProjection(S0)

  % Set the random seed, if given.
  if nargin>0
    rng(S0);
    S=S0;
  else
    S=rng;
  end
  
  % true to keep only the samples that fullfil triangular inequality.
  % If true, there will be an error since the projection procedure only
  % consideres the triangular inequality partially. This is used to
  % evaluate the overestimation when we ignore the triangular inequality.
  triangular=false;
  
  % Number of points
  P.n=4;
  
  % dimension of the embedding space
  P.dim=2;
  
  % Random points
  X=rand(P.n,P.dim);
  
  % Distance matrix
  D=squareform(pdist(X)).^2;
  
  % Variable entries in the distance matrix
  P.V=[0 1 1 1
       1 0 1 1
       1 1 0 1
       1 1 1 0];
     
  % enlarge lower limit of variable distances
  T=tril(rand(P.n));
  R=T+T';
  P.L=D-R.*P.V;
  % Lower can not be negative
  P.L(P.L<0)=0;
  
  % enlarge upper limit of variable distances
  T=tril(rand(P.n));
  R=T+T';
  P.U=D+R.*P.V;
  
  % Avoid 0 in P_1_n
  if P.L(1,P.n)==0
    P.L(1,P.n)=P.L(1,P.n)+0.01;
    P.U(1,P.n)=P.L(1,P.n)+0.01;
    P.L(P.n,1)=P.L(P.n,1)+0.01;
    P.U(P.n,1)=P.L(P.n,1)+0.01;
  end
  
  % Apply triangular ineguality
  [P,empty]=triangIneq(P);
  
  % If the problem is still feasible
  if ~empty
    
    % Compute the projection
    [V,O]=project(P);
    
    % Now sample the variable ranges and project
    VS.L=inf(P.n);
    VS.U=-inf(P.n);
    OS.L=inf(P.n);
    OS.U=-inf(P.n);
    m=10000*sum(sum(P.V)); % number of samples
    for k=1:m
      D1=sampleProblem(P);
      if triangular
        P1=P;
        P1.L=D1;
        P1.U=D1;
        [~,empty]=triangIneq(P1);
      else
        empty=false;
      end
      if ~empty
        V1=zeros(P.n);
        O1=zeros(P.n);
        for i=1:P.n
          for j=1:P.n
            V1(i,j)=(D1(i,P.n)-D1(j,P.n)+D1(j,1)-D1(i,1))/(2*sqrt(D1(1,P.n)));
            O1(i,j)=max(0,D1(i,j)-V1(i,j)^2);
          end
        end
        VS.L=min(VS.L,V1);
        VS.U=max(VS.U,V1);
        OS.L=min(OS.L,O1);
        OS.U=max(OS.U,O1);
      end
    end
    
    % Compare the two methods
    e1=max(max(abs(V.L-VS.L)))/max(max(abs(V.U-V.L)));
    e2=max(max(abs(V.U-VS.U)))/max(max(abs(V.U-V.L)));
    
    e3=max(max(abs(O.L-OS.L)))/max(max(abs(O.U-O.L)));
    e4=max(max(abs(O.U-OS.U)))/max(max(abs(O.U-O.L)));
    
    e=max([e1 e2 e3 e4]);
  else
    e=0;
  end
  
  