% Propagates the reduction in O to V. In this propagation we take into
% account that 
%          d_i_j^2 = o_i_j^2 + v_i_j^2
% and, thus
%          v_i_j^2 = d_i_j^2 - o_i_j^2
%          v_i_j = +/- sqrt( d_i_j^2 - o_i_j^2 )
%
% Note that we already have the squared d and o and thus we have
%          v_i_j = +/- sqrt( d_i_j - o_i_j )
%
% the +/- of the sqrt define two intervals and we have to compute the hull
% for these two intervals with the original interval for v_i_j.
%
% For d_1_i we additionally take into account that
%          d_1_i^2 = o_1_i^2 + (v_1_i-d_1_n)^2
% and, thus,
%          v_1_i = +/- sqrt(d_1_i^2 - o_1_i^2) + d_1_n
%
% Here again the +/- sign define two intervals that must be intersected
% with the current range for v_1_i. The final output is the hull fo these
% two intersections (see hullIntersect.m). 
% Note that d_1_n is actually computed as sqrt(d_1_n) since we represent
% problems by squared distances. This sqrt, however, can only be possitive.
%
% For d_i_n we can use the fact that
%          d_i_n^2 = o_i_n^2 + (v_i_n-d_1_n)^2
% and, thus,
%          v_i_n = +/- sqrt(d_i_n^2 - o_i_n^2) + d_1_n
%
% Inputs:
%     P: The original problem (probably permuted).
%     O: The squares of the projection on the orthogonal subspace reduced
%        after applying triangular ineguality.
%     V: The projection on the projection axis (1-n).
%
% Outputs:
%    V1: The reduced projection after propagating the reduction in O to V
%        and after applying the triangular inequality to propagate the
%        possible reductions in any element in V.
% empty: True if the problem is determined unfeaible in the process.
%
function [V1,empty]=O2V(P,O,V)

  roundUp=@()(system_dependent('setround',Inf));
  roundDown=@()(system_dependent('setround',-Inf));
  roundNear=@()(system_dependent('setround','nearest'));

  V1=V;
  
  % general propagation (for any i,j)
  empty=false;
  i=1;
  while ~empty && i<=P.n
    j=i+1;
    while ~empty && j<=P.n
      roundDown();
      l1=sqrt0(P.L(i,j)-O.U(i,j));
      roundUp();
      u1=sqrt0(P.U(i,j)-O.L(i,j));
      roundNear();
      l2=-u1;
      u2=-l1;
      
      [V1.L(i,j),V1.U(i,j),empty]=hullIntersect(O.eps,V1.L(i,j),V1.U(i,j),l1,u1,l2,u2);
        
      V1.L(j,i)=-V1.U(i,j);
      V1.U(j,i)=-V1.L(i,j);
      
      j=j+1;
    end
    i=i+1;
  end
  
  % particular propagation for d_1_i
  i=2;
  while ~empty && i<P.n
    roundDown();
    a1=sqrt0(P.L(i,P.n)-O.U(1,i));
    a2=sqrt0(P.L(1,P.n));
    l1=a1+a2;
    roundUp();
    b1=sqrt0(P.U(i,P.n)-O.L(1,i));
    b2=sqrt0(P.U(1,P.n));
    u1=b1+b2;
    roundDown();
    l2=-b1+a2;
    roundUp();
    u2=-a1+b2;
    roundNear();
    
    [V1.L(1,i),V1.U(1,i),empty]=hullIntersect(O.eps,V1.L(1,i),V1.U(1,i),l1,u1,l2,u2);
    
    V1.L(i,1)=-V1.U(1,i);
    V1.U(i,1)=-V1.L(1,i);
      
    i=i+1;
  end
  
  % particular propagation for d_i_n
  i=2;
  while ~empty && i<P.n
    roundDown();
    a1=sqrt0(P.L(1,i)-O.U(i,P.n));
    a2=sqrt0(P.L(1,P.n));
    l1=a1+a2;
    roundUp();
    b1=sqrt0(P.U(1,i)-O.L(i,P.n));
    b2=sqrt0(P.U(1,P.n));
    u1=b1+b2;
    roundDown();
    l2=-b1+a2;
    roundUp();
    u2=-a1+b2;
    roundNear();
    
    [V1.L(i,P.n),V1.U(i,P.n),empty]=hullIntersect(O.eps,V1.L(i,P.n),V1.U(i,P.n),l1,u1,l2,u2);
    
    V1.L(P.n,i)=-V1.U(i,P.n);
    V1.U(P.n,i)=-V1.L(i,P.n);
    
    i=i+1;
  end
  
  if ~empty
    [V1,empty]=triangEq(V1,O.eps);
  end
  
  