% Reconstruct the problem from the projections. The backprojection is based
% in the fact that 
%      d_i_j^2 = o_i_j^2 + v_i_j^2
% where d_i_j are the distances in the original problem, o_i_j are
% the distances in the orthogonal plane and v_i_j are the 
% projections on the projection axis. 
% Note that in our problem representation both d_i_j  and o_i_j are already
% squared.
%
% This is a critical step in this solver: Accurate (i.e., non-expansive)
% backprojections result if large reductios of the boxes.
%
% For distance d_1_i, is additionally computed using d_1_n and d_i_n using
% the cosine rule:
%       d_1_i^2 = d_1_n^2 + d_i_n^2 - 2*d_1_n*v_i_n
%
% Observe that v_i_n=d_i_n*cos(a) where is the angle between d_i_n and the
% projection axis. This alternative backprojection combined with the
% previous one produces a might tighter backprojection. Here d_1_i is
% computed from v_i_n and this crosses information between different
% distances, which is crucial to obtain tighter bounds. Clearly, this only
% procudues a tight backprojection if v_i_n is reduced with respect to the
% original projection (see O2V).
%
% A similar reasoning can be done for d_i_n using d_1_n and d_1_n:
%       d_i_n^2 = d_1_n^2 + d_1_i^2 - 2*d_1_i*v_1_i
%
% Inputs:
%     P: The original problem. The backprojection is expansive and we have
%        to avoid returning ranges larger than the original ones.
%     V: The coordinates on the projection axis-
%     O: The squared distances in the subspace orthogonal to the projectin
%        axis.
% Outputs:
%    P!: The backprojected problem.
% empty: true if the problem is determined to be unfeasible during the
%        backprojection.
%
function [P1,empty]=backprojection(P,V,O)

  roundUp=@()(system_dependent('setround',Inf));
  roundDown=@()(system_dependent('setround',-Inf));
  roundNear=@()(system_dependent('setround','nearest'));

  empty=false;
  P1=P;
  
  for i=1:P1.n
    for j=i+1:P1.n
      if P1.V(i,j)
        [a,b]=square(V.L(i,j),V.U(i,j));
        roundDown();
        l=a+O.L(i,j);
        roundUp();
        u=b+O.U(i,j);
        roundNear();
  
        P1.L(i,j)=max(P1.L(i,j),l-P.eps);
        P1.U(i,j)=min(P1.U(i,j),u+P.eps);
        
        P1.L(j,i)=P1.L(i,j);
        P1.U(j,i)=P1.U(i,j);
      end
    end
  end
  
  % special treatment for d(1,i)
  roundDown();
  d1nL=sqrt(P1.L(1,P1.n));
  roundUp();
  d1nU=sqrt(P1.U(1,P1.n));
  roundNear();
      
  i=2;
  while ~empty && i<P1.n
    if P1.V(1,i)
      mi=[];
      ma=[];
      
      if d1nL<V.U(i,P1.n)
        % derivative w.r.t d_1_n can be negative
        roundUp();
        a=2*d1nU*V.U(i,P1.n);
        roundDown();
        l=P1.U(1,P1.n)+P1.L(i,P1.n)-a;
        a=2*d1nL*V.L(i,P1.n);
        roundUp();
        u=P1.L(1,P1.n)+P1.U(i,P1.n)-a;
        mi=[mi l];
        ma=[ma u];
      end
      
      if V.L(i,P1.n)<d1nU
        % derivative w.r.t d_1_n can be possitive
        roundUp();
        a=2*d1nL*V.U(i,P1.n);
        roundDown();
        l=P1.L(1,P1.n)+P1.L(i,P1.n)-a;
        a=2*d1nU*V.L(i,P1.n);
        roundUp();
        u=P1.U(1,P1.n)+P1.U(i,P1.n)-a;
        mi=[mi l];
        ma=[ma u];
      end
      
      if d1nL<V.U(i,P1.n) &&  V.L(i,P1.n)<d1nU
        % derivative can be both + and - -> can be 0
        roundDown();
        l=P1.L(i,P1.n)-P1.U(1,P1.n);
        roundUp();
        u=P1.U(i,P1.n)-P1.L(1,P1.n);
        mi=[mi l];
        ma=[ma u];
      end
      roundNear();
      
      P1.L(1,i)=max(P1.L(1,i),min(mi)-P.eps);
      P1.U(1,i)=min(P1.U(1,i),max(ma)+P.eps);
      
      empty=P1.L(1,i)>P1.U(1,i);
      
      P1.L(i,1)=P1.L(1,i);
      P1.U(i,1)=P1.U(1,i);
    end
    i=i+1;
  end
 
  % special treatment for d(i,P1.n)
  i=2;
  while ~empty && i<P1.n
    if P1.V(i,P1.n)
      mi=[];
      ma=[];
      
      if d1nL<V.U(1,i)
        % derivative w.r.t d_1_n can be negative
        roundUp();
        a=2*d1nU*V.U(1,i);
        roundDown();
        l=P1.U(1,P1.n)+P1.L(1,i)-a;
        a=2*d1nL*V.L(1,i);
        roundUp();
        u=P1.L(1,P1.n)+P1.U(1,i)-a;
        mi=[mi l];
        ma=[ma u];
      end
      
      if V.L(1,i)<d1nU
        % derivative w.r.t d_1_n can be possitive
        roundUp();
        a=2*d1nL*V.U(1,i);
        roundDown();
        l=P1.L(1,P1.n)+P1.L(1,i)-a;
        a=2*d1nU*V.L(1,i);
        roundUp();
        u=P1.U(1,P1.n)+P1.U(1,i)-a;
        mi=[mi l];
        ma=[ma u];
      end
      
      if d1nL<V.U(1,i) &&  V.L(i,P1.n)<d1nU
        % derivative can be both + and - -> can be 0
        roundDown();
        l=P1.L(1,i)-P1.U(1,P1.n);
        roundUp();
        u=P1.U(1,i)-P1.L(1,P1.n);
        mi=[mi l];
        ma=[ma u];
      end
      roundNear();
      
      P1.L(i,P1.n)=max(P1.L(i,P1.n),min(mi)-P.eps);
      P1.U(i,P1.n)=min(P1.U(i,P1.n),max(ma)+P.eps);
      
      empty=P1.L(i,P1.n)>P1.U(i,P1.n);
      
      P1.L(P1.n,i)=P1.L(i,P1.n);
      P1.U(P1.n,i)=P1.U(i,P1.n);
    end
    i=i+1;
  end
  
  if ~empty
    [P1,empty]=triangIneq(P1);
  end
  