% Computes the hull of the intersection of an interval with two other
% intervals:
%
%    [nl,nu]=hull(intersection([l,u],[l1,u1]),intersection([l,u],[l2,u2]))
%
% This is a basic operation appearing, for instance, in triangIneq2.
%
% Inputs:
%   eps: Numerical accuracy.
%     l: Lower limit of the interval.
%     u: Upper limit of the interval.
%    l1: Lower limit of the first interval.
%    u1: Upper limit of the first interval.
%    l2: Lower limit of the second interval.
%    u2: Upper limit of the second interval.
%
% Output:
%    nl: The lower limit of the hull.
%    nu: The upper limit of the hull.
% empty: True if the hull is empty.
%     s: One if the two intersections are disjoint. This suggest that a
%        bisection might separate two solutions.
%
function [nl,nu,empty,s]=hullIntersect(eps,l,u,l1,u1,l2,u2)

  s=0;
  empty=false;

  % intersect [l,u] [l1,u1]
  l1=max(l,l1-eps);
  u1=min(u,u1+eps);
  
  % intersect [l,u] [l2,u2]
  l2=max(l,l2-eps);
  u2=min(u,u2+eps);
  
  % hull of the two intersections (if not empty)
  if l1<=u1 && l2<=u2
    
    % hull of the two non-empty intervals
    nl=min(l1,l2);
    nu=max(u1,u2);
    
    % If there is a hole in between the to hull-ed intervals.
    if u1<l2 || u2<l1
      s=1;
    end
  else
    % only the first intersection is not empty
    if l1<=u1
      nl=l1;
      nu=u1;
    else
      % only the second intersection is not empty
      if l2<=u2
        nl=l2;
        nu=u2;
      else
        % the two are empty
        nl=u;
        nu=l;
        empty=true;
      end
    end
  end
  