% Projects a problem. 
% We define a projection axis using points 1 and n and project all the
% points into this axis (defining V) and into the subspace orthogonal to
% this axis (defining O).
%
% This is the critical procedure of ths solver: Accurate projections result
% in rapid reduction of the boxes toward the solution points. 
% In the original ICRA05 pape we used interval arithmetic to compute the
% projections. This is a simple procedure but results in large
% overestimation of the projections. Here we use a monotoniciaty analysis
% of the projection formulae and, thus, we obtain tighter bounds. Even
% tighter bound can be obtained if we take into account the triangular
% ineguality. However this results in a significantly more complex
% procedure and in our experience, this complexity is not justified: the
% bounds we obtain right now are good enough. Thus, the projection problem
% has a trade off between complexity and accuracy.
%
% Inputs: 
%     P: The problem with the lower and upper bounds for the squared 
%        distances.
%
% Outputs:
%     V: The projection into the projection axis.
%     O: The projection into subspace orthogonal to the projection axis.
%        This returned as squared distances in a structure identical to P
%        and, thus, the procedure can be recursively applied to  O.
% empty: True if the problem is determined unfeasible 
% error: True if an error is detected during the projection. This only
%        happens when the distance between points 1, n defining the
%        projection axis can be 0 (and, thus, the projections can not be
%        defined).
%
function [V,O,empty,error]=project(P)
  
  roundUp=@()(system_dependent('setround',Inf));
  roundDown=@()(system_dependent('setround',-Inf));
  roundNear=@()(system_dependent('setround','nearest'));
  
  empty=false;

  % signed projection on line 1->n
  V.L=zeros(P.n);
  V.U=zeros(P.n);
  V.n=P.n;
  V.V=P.V;
  V.dim=1;
    
  % d_1_n is used without square in many places
  roundDown();
  dl_1_n=sqrt(P.L(1,P.n));
  roundUp();
  du_1_n=sqrt(P.U(1,P.n));
  roundNear();
  
  if dl_1_n<=0
    error=1;
    V=-1;
    O=-1;
  else
    error=0;
    
    % V(i,P.n)
    for i=2:P.n-1
      mi=[];
      ma=[];
      
      roundDown();
      l=P.L(i,P.n)-P.U(1,i)-P.U(1,P.n);
      roundUp();
      u=P.U(i,P.n)-P.L(1,i)-P.L(1,P.n);
      
      if l<0
        roundUp();
        b1=(2*dl_1_n);
        a2=(P.U(1,P.n)+P.U(i,P.n)-P.L(1,i));
        roundDown();
        a1=(P.L(1,P.n)+P.L(i,P.n)-P.U(1,i));
        b2=(2*du_1_n);
        mi=[mi a1/b1];
        roundUp();
        ma=[ma a2/b2];
      end
      if u>0
        roundUp();
        b1=(2*du_1_n);
        a2=(P.L(1,P.n)+P.U(i,P.n)-P.L(1,i));
        roundDown();
        a1=(P.U(1,P.n)+P.L(i,P.n)-P.U(1,i));
        b2=(2*dl_1_n);
        mi=[mi a1/b1];
        roundUp();
        ma=[ma a2/b2];
      end
      if l<=0 && u>=0
        roundDown();
        mi=[mi max(dl_1_n,sqrt0(P.L(i,P.n)-P.U(1,i)))];
        roundUp();
        ma=[ma min(du_1_n,sqrt0(P.U(i,P.n)-P.L(1,i)))];
      end
     
      % Apply triangular inequality to d_i_n<=d_i_1+d_1_n to
      % define extremes (just in case triangl ineq. not applied
      % beforehand).
      roundDown();
      l=max(-sqrt(P.U(i,P.n)),dl_1_n-sqrt(P.U(1,i)));
      roundUp();
      u=min(+sqrt(P.U(i,P.n)),du_1_n+sqrt(P.U(1,i)));
      roundNear();
      
      %l=-inf;
      %u=+inf;
      
      V.L(i,P.n)=max(l,min(mi))-P.eps;
      V.U(i,P.n)=min(u,max(ma))+P.eps;
    end
    
    % V(1,j)
    for j=2:P.n-1
      mi=[];
      ma=[];
      
      roundDown();
      l=P.L(1,j)-P.U(j,P.n)-P.U(1,P.n);
      roundUp();
      u=P.U(1,j)-P.L(j,P.n)-P.L(1,P.n);
      
      if l<0
        roundUp();
        b1=(2*dl_1_n);
        a2=(P.U(1,P.n)+P.U(1,j)-P.L(j,P.n));
        roundDown();
        a1=(P.L(1,P.n)+P.L(1,j)-P.U(j,P.n));
        b2=(2*du_1_n);
        mi=[mi a1/b1];
        roundUp();
        ma=[ma a2/b2];
      end
      if u>0
        roundUp();
        b1=(2*du_1_n);
        a2=(P.L(1,P.n)+P.U(1,j)-P.L(j,P.n));
        roundDown();
        a1=(P.U(1,P.n)+P.L(1,j)-P.U(j,P.n));
        b2=(2*dl_1_n);
        mi=[mi a1/b1];
        roundUp();
        ma=[ma a2/b2];
      end
      if l<=0 && u>=0
        roundDown();
        mi=[mi max(dl_1_n,sqrt0(P.L(i,P.n)-P.U(j,P.n)))];
        roundUp();
        ma=[ma min(du_1_n,sqrt0(P.U(i,P.n)-P.L(j,P.n)))];
      end
           
      % Apply triangular inequality to d_1_j<=d_1_n+d_n_j to
      % define extremes (just in case triangl ineq. not applied
      % beforehand).
      roundDown();
      l=max(-sqrt(P.U(1,j)),dl_1_n-sqrt(P.U(j,P.n)));
      roundUp();
      u=min(+sqrt(P.U(1,j)),du_1_n+sqrt(P.U(j,P.n)));
      roundNear();
            
      %l=-inf;
      %u=+inf;
      
      V.L(1,j)=max(l,min(mi))-P.eps;
      V.U(1,j)=min(u,max(ma))+P.eps;
      
      % Clearly V(1,j)+V(j,n)=d_1_n -> V(1,j)=d_1_n-V(j,n)
      % This is again triangular ineq. but on the projection, not on the
      % extreme values.
      roundDown();
      l=dl_1_n-V.U(j,P.n);
      roundUp();
      u=du_1_n-V.L(j,P.n);
      roundNear();
      
      V.L(1,j)=max(V.L(1,j),l)-P.eps;
      V.U(1,j)=min(V.U(1,j),u)+P.eps;
    end
    
    % V(1,P.n)
    V.L(1,P.n)=dl_1_n;
    V.U(1,P.n)=du_1_n;
    
    % V(i,j)
    for i=2:P.n-1
      for j=i+1:P.n-1
        mi=[];
        ma=[];
        
        roundDown();
        l=P.L(1,j)-P.U(j,P.n)+P.L(i,P.n)-P.U(1,i);
        roundUp();
        u=P.U(1,j)-P.L(j,P.n)+P.U(i,P.n)-P.L(1,i);
        
        if l<=0
          % derivative w.r.t. d_1_n can be possitive
          roundUp();
          b1=(2*dl_1_n);
          roundDown();
          b2=(2*du_1_n);
          mi=[mi l/b1];
          roundUp();
          ma=[ma u/b2];
        end
        
        if u>=0
          % derivative w.r.t. d_1_n can be negative
          roundUp();
          b1=(2*du_1_n);
          roundDown();
          b2=(2*dl_1_n);
          mi=[mi l/b1];
          roundUp();
          ma=[ma u/b2];
        end
        roundNear();
        
        V.L(i,j)=min(mi)-P.eps;
        V.U(i,j)=max(ma)+P.eps;
      end
    end
    
    L=V.L;
    V.L=V.L-V.U';
    V.U=V.U-L';
    e=find(V.L>V.U);
    if size(e,1)>0
      empty=true;
    end
    
    
    % projection on the sub.space orthogonal to 1->n
    % O(i,P.n)
    O=P; % O must be a full problem, with all the fields
    O.L=zeros(P.n);
    O.U=zeros(P.n);
    O.n=P.n-1;
    O.dim=P.dim-1;
    
    % O(i,P.n)
    % This is the same as O(i,1)=O(1,i). It is computed only to have
    % alternative values for O(1,i).
    % TODO: remove this.
    for i=2:P.n-1
      mi=[];
      ma=[];
      
      roundDown();
      l=P.L(i,P.n)-P.U(1,i);
      roundUp();
      u=P.U(i,P.n)-P.L(1,i);
      
      if l<-P.L(1,P.n)
        roundUp();
        b1=(P.L(i,P.n)-P.U(1,i)+P.L(1,P.n))^2;
        roundDown();
        c1=(4*P.L(1,P.n));
        mi=[mi P.L(i,P.n)-b1/c1];
        b2=(P.U(i,P.n)-P.L(1,i)+P.U(1,P.n))^2;
        roundUp();
        c2=(4*P.U(1,P.n));
        ma=[ma P.U(i,P.n)-b2/c2];
      end
      
      if l<P.U(1,P.n) && u>-P.U(1,P.n)
        roundUp();
        b1=(P.L(i,P.n)-P.L(1,i)+P.U(1,P.n))^2;
        roundDown();
        c1=(4*P.U(1,P.n));
        mi=[mi P.L(i,P.n)-b1/c1];
        b2=(P.U(i,P.n)-P.U(1,i)+P.L(1,P.n))^2;
        roundUp();
        c2=(4*P.L(1,P.n));
        ma=[ma P.U(i,P.n)-b2/c2];
      end
      
      if u>P.L(1,P.n)
        roundUp();
        b1=(P.U(i,P.n)-P.L(1,i)+P.L(1,P.n))^2;
        roundDown();
        c1=(4*P.L(1,P.n));
        mi=[mi P.U(i,P.n)-b1/c1];
        b2=(P.L(i,P.n)-P.U(1,i)+P.U(1,P.n))^2;
        roundUp();
        c2=(4*P.U(1,P.n));
        ma=[ma P.L(i,P.n)-b2/c2];
      end
      
      if l<=-P.L(1,P.n) && u>=-P.U(1,P.n)
        roundDown();
        mi=[mi max(P.L(i,P.n),P.L(1,i)-P.U(1,P.n))];
        roundUp();
        ma=[ma min(P.U(i,P.n),P.U(1,i)-P.L(1,P.n))];
      end
      
      if l<=P.U(1,P.n) && u>=P.L(1,P.n)
        roundDown();
        mi=[mi max(P.L(1,i),P.L(i,P.n)-P.U(1,P.n))];
        roundUp();
        ma=[ma min(P.U(1,i),P.U(i,P.n)-P.L(1,P.n))];
      end
      roundNear();
      
      [vl2,vu2]=square(V.L(i,P.n),V.U(i,P.n));
      
      O.L(i,P.n)=max([0 P.L(i,P.n)-vu2-P.eps min(mi)-P.eps]);
      O.U(i,P.n)=min(   P.U(i,P.n)-vl2+P.eps,max(ma)+P.eps);
      
    end
    
    % O(1,j)
    for j=2:P.n-1
      mi=[];
      ma=[];
      
      roundDown();
      l=P.L(1,j)-P.U(j,P.n);
      roundUp();
      u=P.U(1,j)-P.L(j,P.n);
      
      if l<-P.L(1,P.n)
        roundUp();
        b1=(P.L(1,j)-P.U(j,P.n)+P.L(1,P.n))^2;
        roundDown();
        c1=(4*P.L(1,P.n));
        mi=[mi P.L(1,j)-b1/c1];
        b2=(P.U(1,j)-P.L(j,P.n)+P.U(1,P.n))^2;
        roundUp();
        c2=(4*P.U(1,P.n));
        ma=[ma P.U(1,j)-b2/c2];
      end
      
      if l<P.U(1,P.n)&& u>-P.U(1,P.n)
        roundUp();
        b1=(P.L(1,j)-P.L(j,P.n)+P.U(1,P.n))^2;
        roundDown();
        c1=(4*P.U(1,P.n));
        mi=[mi P.L(1,j)-b1/c1];
        b2=(P.U(1,j)-P.U(j,P.n)+P.L(1,P.n))^2;
        roundUp();
        c2=(4*P.L(1,P.n));
        ma=[ma P.U(1,j)-b2/c2];
      end
      
      if u>P.L(1,P.n)
        roundUp();
        b1=(P.U(1,j)-P.L(j,P.n)+P.L(1,P.n))^2;
        roundDown();
        c1=(4*P.L(1,P.n));
        mi=[mi P.U(1,j)-b1/c1];
        b2=(P.L(1,j)-P.U(j,P.n)+P.U(1,P.n))^2;
        roundUp();
        c2=(4*P.U(1,P.n));
        ma=[ma P.L(1,j)-b2/c2];
      end
      
      if l<=-P.L(1,P.n) && u>=-P.U(1,P.n)
        roundDown();
        mi=[mi max(P.L(1,j),P.L(j,P.n)-P.U(1,P.n))];
        roundUp();
        ma=[ma min(P.U(1,j),P.U(j,P.n)-P.L(1,P.n))];
      end
      
      if l<=P.U(1,P.n)&& u>=P.L(1,P.n)
        roundDown();
        mi=[mi max(P.L(j,P.n),P.L(1,j)-P.U(1,P.n))];
        roundUp();
        ma=[ma min(P.U(j,P.n),P.U(1,j)-P.L(1,P.n))];
      end
      roundNear();
      
      [vl2,vu2]=square(V.L(1,j),V.U(1,j));
      
      O.L(1,j)=max([0 P.L(1,j)-vu2-P.eps min(mi)-P.eps]);
      O.U(1,j)=min(   P.U(1,j)-vl2+P.eps,max([0 ma])+P.eps);
      
      % Clearly O_1_j must be equal to O_j_n -> intersect the two computed
      % ranges
      l=max(0,max(O.L(1,j),O.L(j,P.n))-P.eps);
      u=min(O.U(1,j),O.U(j,P.n))+P.eps;
      
      O.L(1,j)=l;
      O.U(1,j)=u;
      
      O.L(j,P.n)=l;
      O.U(j,P.n)=u;
    end
    
    % just for clarity (they are already 0)
    O.L(1,P.n)=0;
    O.U(1,P.n)=0;
    
    % O(i,j)
    for i=2:P.n-1
      for j=i+1:P.n-1
        mi=[];
        ma=[];
        
        roundDown();
        l=P.L(1,j)-P.U(j,P.n)+P.L(i,P.n)-P.U(1,i);
        roundUp();
        u=P.U(1,j)-P.L(j,P.n)+P.U(i,P.n)-P.L(1,i);
        
        if l<0
          roundUp();
          l2=l^2;
          roundDown();
          c1=(4*P.L(1,P.n));
          mi=[mi P.L(i,j)-l2/c1];
          u2=u^2;
          roundUp();
          c2=(4*P.U(1,P.n));
          ma=[ma P.U(i,j)-u2/c2];
        end
        
        if u>0
          roundUp();
          u2=u^2;
          roundDown();
          c1=(4*P.L(1,P.n));
          mi=[mi P.L(i,j)-u2/c1];
          l2=l^2;
          roundUp();
          c2=(4*P.U(1,P.n));
          ma=[ma P.U(i,j)-l2/c2];
        end
        
        if l<=0 && u>=0
          roundDown();
          mi=[mi P.L(i,j)];
          roundUp();
          ma=[ma P.U(i,j)];
        end
        roundNear();
        
        [vl2,vu2]=square(V.L(i,j),V.U(i,j));
        
        O.L(i,j)=max([0 P.L(i,j)-vu2-P.eps min(mi)-P.eps]);
        O.U(i,j)=min(   P.U(i,j)-vl2+P.eps,max(ma)+P.eps);
      end
    end
    
    O.L=O.L+O.L';
    O.U=O.U+O.U';
    % Sincd point 1 and n are the same, we can remove point n
    r=1:O.n;
    O.L=O.L(r,r);
    O.U=O.U(r,r);
    O.V=O.V(r,r);
    
    e=find(O.L>O.U);
    if size(e,1)>0
      empty=true;
    end
    
  end
  