% Reduces a box defined by the ranges bounding the solutions of the 
% problem at hand. 
% Those ranges are stored in the structure P with the lower and upper
% bounds for the squared distances between points. P also includes other
% parameters, though. The ranges are reduced by projecting the inter-point
% distances to a given axis and its orthgonal subspace, reducing the ranges
% in this subsapce, propagating the eventual reductions to the projections
% on the projection axis and, finally, backprojecting to determine tighter
% bound in the original space.
% The projection axis are defined by two point in the problem and the
% different couples to consider are stored in P.
%
% Inputs:
%     P: The problem with the ranges to reduce.
%
% Outputs:
%     P: The problem with the possibly reduced ranges.
% empty: True if the problem is determined to be unfeasible, i.e, the given
%        box is empty.
%     s: A matrix indicating which ranges should be used to split the
%        problem, i.e,. ranges than when split are likely to separate two
%        solutions.
%    np: Number of projections actually computed. Used to gather statistics
%        about the solution process.
%
% 
function [P,empty,s,np,red,nRed]=reduceBox(P,red,nRed)
  empty=false;
  s=zeros(P.n);
  np=1;
  while ~empty && np<=P.nProj

    sz=sum(sum(P.U-P.L))/2;
    
      [P,empty,er,s1]=oneProjection(P,np,1);
    
    if ~er
      if empty
        nsz=0;
      else
        nsz=sum(sum(P.U-P.L))/2;
      end
      red(np)=red(np)+(nsz/sz);
      nRed(np)=nRed(np)+1;
    end
    
    if ~empty && ~er
      s=s+s1;
    end
    np=np+1;
  end
  np=np-1; % numb. of projections actually performed.  
end

% Auxiliary function to reduce the for for one projection. It can
% recursively call itself for problems of dim=3 where two projections are
% necessary to reach dim==1 where triangular inequality can be used to crop
% ranges.
%
% Inputs:
%     P: The problem to be reduced with the selected projection
%    np: The identifier of the projection.
% level: Recursion level
%
% Outputs:
%    P1: The possibly reduced problem.
% empty: True if the problem is determined to be unfeasible.
%    er: True if there is an error in the process.
%    s1: A matrix indicating which ranges should be used to split the
%        problem, i.e,. ranges than when split are likely to separate two
%        solutions according to the selected projection.
%
function [P1,empty,er,s1]=oneProjection(P,np,level)
  
  % copy the input
  P1=P;
  
  % Points defining the projection axis
  i=P1.proj{np}(2*level-1);
  j=P1.proj{np}(2*level);
  
  % permute
  P1=permute1n(P1,false,level,np,i,j);
  
  % project
  [V,O,empty,er]=project(P1);
  
  if ~empty && ~er
    % reduce O
    if O.dim>1 
      % Recursive projection
      [Ot,empty,er,s1]=oneProjection(O,np,level+1);
    else
      % O.dim==1 directly apply triangular equality (with sign if
      % necessary).
      [Ot,empty,s1]=triangEq2(O,P1.sg{np});
    end
    % TODO: if O==Ot, do not call O2V nor backprojection.
    if ~empty
      % Add the missing point to Ot
      [Ot,s1]=addMissingPoint(Ot,s1);
      
      % translate the reduction to V
      [Vt,empty]=O2V(P1,Ot,V); % includes triangEq(Vt)
      %Vt=triangEq(V);
      %Vt=V;
      if ~empty
        % Backprocject.
        [P1,empty]=backprojection(P1,Vt,Ot);
      end
    end
  else
    s1=zeros(P1.n); % default output
  end
  
  %unpermute
  if ~empty
    [P1,ndx]=permute1n(P1,true,level,np,i,j);
    s1=s1(ndx,ndx);
  end
end
  