% Isolates the solutions of a given problem.
%
% This is the main function of the solver.
%
% We apply the reduceBox process as far as the reduction is larger than a
% given parameter (P.rho). At the end of the reduction, if the box is
% small (smaller than P.sigma), it is considered a solution, otherwise it
% is split an the recursively processed. This recursion explores a tree of
% boxes. Ideally all the leaves of the tree are solutions. Overestimation
% in the projection/backprojection, though, can procude empty boxes, i.e,
% leaves of the explored tree which include no solution.
%
% Inputs:
%     P: The problem to solve, including the initial ranges and the
%        parameters to use in the process. Since the problem is basically
%        defined by the lower/upper ranges of the interpoint distances, we
%        often refer to P as a (n-dimensional axis-aligned) box.
% Outputs:
%     S: The solution boxes, i.e., boxes smaller than P.sigma.
% 
% Examples of use:
%
%     P=robots;
%     S=solveProblem(P);
%
%     P=fourICRA;
%     S=solveProblem(P);
%     plotSolution(S{1});
%     plotSolution(S{2});
%
%     P=fiveICRAOne;
%     S=solveProblem(P);
%     plotSolution(S{1});
%     plotSolution(S{2});
%
%     P=fiveICRATwo;
%     S=solveProblem(P);
%     plotSolution(S{1});
%     plotSolution(S{2});
%
%     P=fiveICRA;
%     S=solveProblem(P);
%     plotBox(S,0.5,1,3,1,5,2,4);
%
%     P=P3RPR;
%     S=solveProblem(P);
%     plotSolution(S{1});
%     hold on;
%     plotSolution(S{2});
%
%     P=P3RPRRandom(1,0.25);
%     S=solveProblem(P);
%     plotSolution(S{1});
%     hold on;
%     plotSolution(S{2});
%     ...
%
function [S,red]=solveProblem(P)
  
  red=zeros(1,P.nProj);
  nRed=zeros(1,P.nProj);
  
  tic;
  L={P}; % Boxes to process
  S={};  % Solution boxes
  npb=0; % number of processed boxes
  neb=0; % number of empty boxes
  np=0;  % number of projections
  nbr=0; % number of box reductions
  nbb=0; % number of bisected boxes
  
  
  % While we have boxes to process
  while size(L,2)>0
    fprintf('Pending boxes: %u    Solutions: %u    Empty: %u\n',size(L,2),size(S,2),neb);
    % Gent the first box. In this way explore the tree breadth-first. Take
    % the last to explore it deep-first.
    P=L{1};
    L=L(2:end);
    npb=npb+1;
    
    % Reduce the box as far as possible, while it is not a solution and not
    % empty.
    empty=false;
      
    sz=sum(sum(P.U-P.L))/2;
    fprintf('      size: %f\n',sz);
    reduce=true;
    small=false;
    while ~empty && ~small && reduce
      nbr=nbr+1;
      
      [P,empty,s,np1,red,nRed]=reduceBox(P,red,nRed);
      np=np+np1;
      
      if empty
        neb=neb+1;
      else
        nsz=sum(sum(P.U-P.L))/2;
        fprintf('      size: %f\n',nsz);
        reduce=(nsz/sz<P.rho);
        sz=nsz;
        small=(sz<P.sigma);
      end
    end
    
    if ~empty
      if small
        % The box is a solution
        S{end+1}=P;
      else
        % The box is not a solution but it is not empty -> split it.
        nbb=nbb+1;
        [P1,P2]=splitBox(P,s);
        L{end+1}=P1;
        L{end+1}=P2;
      end
    end
  end
  
  red=red./nRed;
  
  % Pring some data about the solution process.
  fprintf('Problem solved\n');
  fprintf('  Number of processed boxes: %u\n',npb);
  fprintf('  Number of empty boxes    : %u\n',neb);
  fprintf('  Number of solution boxes : %u\n',size(S,2));
  fprintf('  Number of bisected boxes : %u\n',nbb);
  fprintf('  Number of box reductions : %u\n',nbr);
  fprintf('  Number of projections    : %u\n',np);
  fprintf('  Execution time           : %f\n',toc);
  