% Implements the MDS procedure as described in
%
% Yi Shang and Ying Zhang, Localization from Mere Connectivity, MobiHoc?03
%
% Inputs: 
%         P: The problem description.
%         D: The lower and upper limit for each variables.
%
% Outputs:
%         S: The solution matrix.
%    inside: true if S is inside the ranges defined in D.
%
function [S,inside]=solveProblemMDS(P,D)

  % Fixed distances are taken from input (at random in the given ranges)
  M1=zeros(P.n);
  for i=1:P.n
    for j=i+1:P.n
      if ~P.V(i,j)
        M1(i,j)=D.L(i,j)+(D.U(i,j)-D.L(i,j))*rand;
        M1(j,i)=M1(i,j);
      end
    end
  end
  
  % The missing distances are computed via shortest path
  M2=graphallshortestpaths(sparse(M1));
  
  % Define the double centered matrix to decompose. 
  % This matrix form is taken from 
  %    Distance Matrix Reconstruction from Incomplete Distance Information 
  %    for Sensor Network Localization
  % by 
  %    P. Drineas, A. Javed, M. Magdon-Ismail, G. Pandurangan, 
  %    R. Virrankoski and A. Savvides
  O=ones(P.n,1);
  L=eye(P.n)-(1/P.n)*(O*O');
  Q=-(1/2)*L*M2*L;
  
  % Decompose
  [U,d,~]=svd(Q);
  
  % Get the rank-P.dim approximation
  % The coordinates
  X=U(:,1:P.dim)*sqrt(d(1:P.dim,1:P.dim));
  % and the distance matrix
  S1=squareform(pdist(X)).^2;
  
  % Store in the form of a interval solution for compatibility with
  % plotSolution
  S.L=S1;
  S.U=S1;
  S.n=P.n;
  S.dim=P.dim;
  S.V=P.V;
  
  % Is the output matrix inside the input ranges?
  inside=(max(max((D.L<=S1)&(S1<=D.U)))==0);
  