% Applies the triangular equality.
% This is the same as triangEq but here the input ranges are squared and,
% thus, we have to take the square root on each term before applying the
% inequality, and square the result at the end.
%
% Inputs:
%     P: The problem described as a set of squared lower and upper bounds.
%    sg: Matrix with the sigh of the triangles defined from the current
%        projection.
%
% Outputs:
%    P1: The problem with reduced ranges.
% empty: True is the problem is determined to be infeasible.
%     s: Matrix indicating which ranges should be bisected to separate
%        solutions.
%
function [P1,empty,s]=triangEq2(P,sg)

  roundUp=@()(system_dependent('setround',Inf));
  roundDown=@()(system_dependent('setround',-Inf));
  roundNear=@()(system_dependent('setround','nearest'));
  
  if P.dim>1
    error('triang works one dimensinal spaces');
  end
  
  P1=P;
  
  s=zeros(size(P.L,1)); % size of the original problem
  
  empty=false;
  j=1;
  while ~empty && j<=P.n
    i=1;
    while ~empty && i<P.n
      k=i+1;
      while ~empty && k<=P.n
        if P1.U(i,k)-P.L(i,k)>1e-10
          if j>1 || sg(i,k)<=0
            roundDown();
            l=sqrt(P1.L(i,j))-sqrt(P1.U(j,k));
            roundUp();
            u=sqrt(P1.U(i,j))-sqrt(P1.L(j,k));
            
            [mi1,ma1]=square(l,u);
          else
            mi1=+1;
            ma1=-1;
          end
          
          if j>1 || sg(i,k)>=0
            roundDown();
            l=sqrt(P1.L(i,j))+sqrt(P1.L(j,k));
            roundUp();
            u=sqrt(P1.U(i,j))+sqrt(P1.U(j,k));
            
            [mi2,ma2]=square(l,u);
          else
            mi2=+1;
            ma2=-1;
          end
          
          roundNear();
          
          [P1.L(i,k),P1.U(i,k),empty,s1]=hullIntersect(P1.eps,P1.L(i,k),P1.U(i,k),mi1,ma1,mi2,ma2);
          
          if P1.V(i,k)
            s(i,k)=s(i,k)+s1;
            s(k,i)=s(i,k);
          end
          
          P1.L(k,i)=P1.L(i,k);
          P1.U(k,i)=P1.U(i,k);
        end
        k=k+1;
      end
      i=i+1;
    end
    j=j+1;
  end
  
%   if size(P.L,1)>P.n
%     % We are applying the triang. equality to a projection. Point 'P.n+1'
%     % is the same as point 1 -> copy the ranges
%     P1.L(P.n+1,2:P.n)=P1.L(1,2:P.n);
%     P1.U(P.n+1,2:P.n)=P1.U(1,2:P.n);
%     P1.L(2:P.n,P.n+1)=P1.L(2:P.n,1);
%     P1.U(2:P.n,P.n+1)=P1.U(2:P.n,1);
%   end
