% Example with 6 robots on a plane. For the Sensor journal paper.
%
% Inputs:
%    orientation: [optional] true if we have to fix the relative 
%                 orientation of particuar triangles. If not given true is
%                 assumed.
% Outputs:
%     P: The problem structure.
%    D1: Distance matrix as obtanied from sensors.
%    D2: Distance matrix after applying triangular inequality.
%
function [P,D1,D2]=robots(varargin)
  
  if isempty(varargin)
    orientation=true;
  else
    orientation=varargin{1};
  end

  % Nominal positions
  X=[-7 +0 -5 +2 +6 +7
     +3 +0 -4 -7 +4 -2];
  
  P.n=size(X,2);
  P.L=squareform(pdist(X')).^2;
  P.U=P.L;
 
  % use P.V to mark the non-measured distances
  P.V=ones(P.n);
  % 0=measured
  P.V(1,2)=0;P.V(2,1)=0;
  P.V(1,3)=0;P.V(3,1)=0;
  P.V(2,3)=0;P.V(3,2)=0;
  P.V(2,4)=0;P.V(4,2)=0;
  P.V(2,5)=0;P.V(5,2)=0;
  P.V(2,6)=0;P.V(6,2)=0;
  P.V(3,4)=0;P.V(4,3)=0;
  P.V(4,6)=0;P.V(6,4)=0;
  P.V(5,6)=0;P.V(6,5)=0;
  for i=1:P.n
    P.V(i,i)=0;
  end
  P.V=(P.V>0);
  
  % The non-measured
  P.L(P.V)=0^2;
  P.U(P.V)=100^2; % infinity
  
  P.eps=1e-15*max(max(P.U));
  
  rng(101); % fix random seed (to ensure repetitibility)
  % add noise (3%)
  for i=1:P.n
    for j=i+1:P.n
      if ~P.V(i,j) % if measured -> add noise of max 3% of measurament
        n=(0.015*sqrt(P.L(i,j)))^2;
        
        P.L(i,j)=P.L(i,j)-n;
        P.U(i,j)=P.U(i,j)+n;
        
        P.L(j,i)=P.L(i,j);
        P.U(j,i)=P.U(i,j);
      end
    end
  end
    
  P.L(P.V)=P.L(P.V)-P.eps;
  neg=(P.L<0);
  P.L(neg)=0;
  P.U(P.V)=P.U(P.V)+P.eps;
  
  D1=P;
   
  [P,empty]=triangIneq(P);
  D2=P;
  
  if empty
    error('inconsistent problem');
  end
  
  % Below this size the box is a solution
  P.sigma=1.25; 
  % Below this reduction ratio, the box is reduced again
  P.rho=0.9;
  % Split the box along its max. side
  P.maxSize=true;
  
  % Dimension of the embedding space
  P.dim=2;
  
  % The projections to use
  P.proj={ [1 2] [1 3] [1 4] [1 5] [1 6] [2 3] [2 4] [2 5] [2 6] [3 4] [3 5] [3 6] [4 5] [4 6] };
  
  % Fix the orientation of particular triangles
  % In general no orientation is fixed
  Z=zeros(P.n);
  if orientation
    % Set this to +/-1 to select one of the solutions
    % The relative position of robots R1 and R4 w.r.t. the segment R2-R3
    Z14=Z;
    Z14(1,4)=+1;
    Z14(4,1)=+1;
    
    % The relative position of robots R3 and R6 w.r.t. the segment R2-R4
    Z36=Z;
    Z36(3,6)=+1;
    Z36(6,3)=+1;
    
    % The relative position of robots R4 and R5 w.r.t. the segment R2-R6
    Z45=Z;
    Z45(4,5)=+1;
    Z45(5,4)=+1;
    
    % use this to fix the orientations (only 1 solution is obtained)
    P.sg=  {   Z     Z     Z     Z     Z    Z14   Z36    Z    Z45    Z     Z     Z     Z     Z   };
  else
    % no orientation is fixed (then 8 solutions are obtained)
    P.sg=  {   Z     Z     Z     Z     Z     Z     Z     Z     Z     Z     Z     Z     Z     Z   };
  end
  
    
  P.nProj=size(P.proj,2);
